package ssh

import (
	"bufio"
	"bytes"
	"fmt"

	configM "git.loafle.net/overflow/overflow_commons_go/modules/config/model"
	"git.loafle.net/overflow/overflow_probe_container_network/crawler"
	"git.loafle.net/overflow/overflow_probe_container_network/crawler/impl/ssh/client"
	"git.loafle.net/overflow/overflow_probe_container_network/crawler/impl/ssh/parser"
	"github.com/satori/go.uuid"
)

type SSHCrawler struct {
	crawler.Crawlers
}

func (c *SSHCrawler) Name() string {
	return "SSH_CRAWLER"
}

func (c *SSHCrawler) InternalGet(config *configM.Config) (map[string]string, error) {
	sshClient, err := client.New(config.Target)
	if nil != err {
		return nil, err
	}

	itemCount := len(config.Items)
	results := make(map[string]string, 0)
	boundary := uuid.NewV4().String()
	commands := ""

	for i := 0; i < itemCount; i++ {
		switch i {
		case 0:
			commands = config.Items[i].QueryInfo.Query
		default:
			commands = fmt.Sprintf("%s ; echo \"--%s\" ; %s ", commands, boundary, config.Items[i].QueryInfo.Query)
		}
	}
	commands = fmt.Sprintf("%s ; echo \"--%s--\" ", commands, boundary)

	buf, err := sshClient.RunCommand(commands)
	if nil != err {
		return nil, err
	}
	r := bytes.NewReader(buf)
	scanner := bufio.NewScanner(r)

ParseLoop:
	for i := 0; i < itemCount; i++ {
		item := config.Items[i]
		mode := item.QueryInfo.Extend["mode"].(string)
		p := parser.GetParser(mode)
		if nil == p {
			return nil, fmt.Errorf("Container: Parser[%s] is not exist", mode)
		}

		rm, err := p.Parse(scanner)
		if nil != err {
			return nil, err
		}

		if nil != rm {
			for k, v := range rm {
				results[k] = v
			}
		}

		for scanner.Scan() {
			line := scanner.Text()
			if line == "--"+boundary+"--" {
				break ParseLoop
			}

			if line == "--"+boundary {
				break
			}
		}

	}

	return results, nil
}

func NewCrawler() crawler.Crawler {
	c := &SSHCrawler{}
	c.Internal = c

	return c
}
