package parser

import "bufio"

var parsers map[string]Parser

func init() {
	parsers = make(map[string]Parser, 0)

	addParser(&CPUParser{})
	addParser(&MemoryParser{})
}

func addParser(p Parser) {
	parsers[p.Name()] = p
}

func GetParser(name string) Parser {
	p, ok := parsers[name]
	if !ok {
		return nil
	}
	return p
}

type Parser interface {
	Name() string
	Parse(scanner *bufio.Scanner) (map[string]string, error)
}
